@@ -680,8 +680,8 @@ class RedisCluster(Redis):
                 self.refresh_table_asap = True
                 self.connection_pool.nodes.increment_reinitialize_counter()
 
-                node = self.connection_pool.nodes.set_node(e.host, e.port, server_type='master')
-                self.connection_pool.nodes.slots[e.slot_id][0] = node
+                node = self.connection_pool.nodes.get_node(e.host, e.port, server_type='master')
+                self.connection_pool.nodes.move_slot_to_node(e.slot_id, node)
             except TryAgainError as e:
                 log.exception("TryAgainError")
 
@@ -973,17 +973,17 @@ class RedisCluster(Redis):
                 for r in resp:
                     yield r
 
-    def mget(self, keys, *args):
-        """
-        Returns a list of values ordered identically to ``keys``
-
-        Cluster impl:
-            Itterate all keys and send GET for each key.
-            This will go alot slower than a normal mget call in Redis.
-
-            Operation is no longer atomic.
-        """
-        return [self.get(arg) for arg in list_or_args(keys, args)]
+    # def mget(self, keys, *args):
+    #     """
+    #     Returns a list of values ordered identically to ``keys``
+    #
+    #     Cluster impl:
+    #         Itterate all keys and send GET for each key.
+    #         This will go alot slower than a normal mget call in Redis.
+    #
+    #         Operation is no longer atomic.
+    #     """
+    #     return [self.get(arg) for arg in list_or_args(keys, args)]
 
     def mset(self, *args, **kwargs):
         """
